/**
 * @license
 * Copyright Alibaba.com All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/NG-ZORRO/ng-zorro-antd/blob/master/LICENSE
 */
import { ChangeDetectorRef, OnDestroy, OnInit } from '@angular/core';
import { MessageConfig, NzConfigService } from 'ng-zorro-antd/core/config';
import { Subject } from 'rxjs';
import { NzMessageDataFilled, NzMessageDataOptions } from './typings';
export declare class NzMessageContainerComponent implements OnInit, OnDestroy {
    protected cdr: ChangeDetectorRef;
    protected nzConfigService: NzConfigService;
    destroy$: Subject<void>;
    messages: NzMessageDataFilled[];
    config: Required<MessageConfig>;
    top: string | null;
    constructor(cdr: ChangeDetectorRef, nzConfigService: NzConfigService);
    ngOnInit(): void;
    ngOnDestroy(): void;
    /**
     * Create a new message.
     * @param message Parsed message configuration.
     */
    createMessage(message: NzMessageDataFilled): void;
    /**
     * Remove a message by `messageId`.
     * @param messageId Id of the message to be removed.
     * @param userAction Whether this is closed by user interaction.
     */
    removeMessage(messageId: string, userAction?: boolean): void;
    /**
     * Remove all messages.
     */
    removeMessageAll(): void;
    protected updateConfig(): void;
    protected subscribeConfigChange(): void;
    protected updateConfigFromConfigService(): Required<MessageConfig>;
    /**
     * Merge default options and custom message options
     * @param options
     */
    protected mergeMessageOptions(options?: NzMessageDataOptions): NzMessageDataOptions;
}
